local mod = ...

--[[

    Adds fuel as a basic need to the game
    Fuel can be provided by: wood or coal

    Dev to-do list --------------------------------------------------------------------
    
    Check for existence of Dung (Vegetable mod) and add as fuel type if it exists
    Path for buyer that is separate from the seller

]]


-- Slow tree growth  -------------------------------------------------------------------------------------

mod:override({
    Id = "DEFAULT_BALANCING",
    TreeGrowSpeed = 0.0005 -- reduce tree growth by half (default speed is 0.001)
    -- TreeGrowSpeed = 2 -- for testing, causes trees to grow instantly
})



-- Resource Type  -------------------------------------------------------------------------------------

mod:registerEnumValue( "RESOURCE_TYPE", "FUEL_RESOURCE_TYPE" )
mod:registerEnumValue( "RESOURCE_TYPE", "FUEL_RESOURCE_TYPE_WOOD" )
mod:registerEnumValue( "RESOURCE_TYPE", "FUEL_RESOURCE_TYPE_COAL" )



-- Happiness factor -------------------------------------------------------------------------------------
mod:register({
    DataType = "HAPPINESS_FACTOR",
    Id = "HAPPINESS_FACTOR_FUEL",
    Value = -2,
    IsDecaying = true,
    TimeBeforeDecayingSeconds = 120.0,
    DecayingRatePerMinute = 3.5,
    IsCumulative = true,
    IsSolvable = true,
    RemovedOnDecayed = false
})



-- Villager Need Type -------------------------------------------------------------------------------------

mod:register({
    DataType = "VILLAGER_NEED_TYPE",
    Id = "FUEL_NEED",
    NeedTypeName = "FUEL_NEED_NAME",
    NeedTypeList = {"FUEL_RESOURCE_TYPE"},
    -- FillDurationInSeconds=840, -- 420 seems to be the duration of water and food, 2-3 days
    FillDurationInSeconds = 1000, -- 420 seems to be the duration of water and food, 2-3 days
    FillRate = 100,
    GracePeriod = 7,
    HappinessFactor = "HAPPINESS_FACTOR_FUEL",
    ResourceTypeIcon = "ICON_FUEL_NEED"
})



-- Villager need -------------------------------------------------------------------------------------
mod:override({
    Id = "SERF",
    AssetVillagerNeedList = {
        Action = "APPEND", "FUEL_NEED"
    }
})

-- Seems that adding the need to SERF was also making it available to the other statuses, so it was creating the need for two fuels at Commoner level, preventing promotion
-- mod:override({
--     Id = "COMMONER",
--     AssetVillagerNeedList = {
--         Action = "APPEND", "FUEL_NEED"
--     }
-- })

-- mod:override({
--     Id = "CITIZEN",
--     AssetVillagerNeedList = {
--         Action = "APPEND", "FUEL_NEED"
--     }
-- })



-- Add Fuel as a resource type on existing resources -------------------------------------------------------------------------------------
mod:override({
	Id = "WOOD",
    ResourceTypeList = { "RESOURCE", "FUEL_RESOURCE_TYPE", "FUEL_RESOURCE_TYPE_WOOD" },
    VillagerBuyingPrice = { Resource = "GOLD", Quantity = 6 }
})

mod:override({
	Id = "COAL",
	ResourceTypeList = { "RESOURCE", "FUEL_RESOURCE_TYPE", "FUEL_RESOURCE_TYPE_COAL" },
    VillagerBuyingPrice = { Resource = "GOLD", Quantity = 12 }
})



-- Job -------------------------------------------------------------------------------------
mod:register({
	DataType = "JOB",
	Id = "FUELSELLER_JOB",
	JobName = "FUELSELLER_JOB_NAME",
	JobDescription = "FUELSELLER_JOB_DESC",
	CharacterSetup = {
		DataType = "CHARACTER_SETUP",
		IdleAnimation = "MARKET_TENDER"
	},
	IsLockedByDefault = false,
	UseWorkplaceBehavior = true,
    ProductionDelay = 10.0,
    AssetJobProgression = "DEFAULT_JOB_PROGRESSION"
})

mod:override({
    Id = "NEWCOMER",
    CompatibleJobList = {
        Action = "APPEND",
        "FUELSELLER_JOB"
    }
})

mod:override({
    Id = "SERF",
    CompatibleJobList = {
        Action = "APPEND",
        "FUELSELLER_JOB"
    }
})

mod:override({
    Id = "COMMONER",
    CompatibleJobList = {
        Action = "APPEND",
        "FUELSELLER_JOB"
    }
})



-- Assets -------------------------------------------------------------------------------------
mod:registerAssetId("icons/fuel.png", "ICON_FUEL_NEED")
mod:registerAssetId("models/FuelCart.fbx/Prefab/FuelCartPart", "FUELCART_PREFAB")



-- Building Asset Processor
mod:registerAssetProcessor("models/FuelCart.fbx", { DataType = "BUILDING_ASSET_PROCESSOR" })



-- Collider Platform (climbable mesh)  -------------------------------------------------------------------------------------
-- mod:configurePrefabFlagList("models/FuelCart.fbx/Prefab/FuelCartPart", { "PLATFORM" })



-- Grounding  -------------------------------------------------------------------------------------
mod:registerPrefabComponent("models/FuelCart.fbx/Prefab/FuelCartPart", { DataType = "COMP_GROUNDED" })
-- mod:registerPrefabComponent("models/FuelCart.fbx/Prefab/FuelCartPart/step_0_Ground", { DataType = "COMP_GROUNDED" })
-- mod:registerPrefabComponent("models/FuelCart.fbx/Prefab/FuelCartPart/step_0_Wagon", { DataType = "COMP_GROUNDED" })
mod:registerPrefabComponent("models/FuelCart.fbx/Prefab/FuelCartPart/step_1_Stump", { DataType = "COMP_GROUNDED" })
-- mod:registerPrefabComponent("models/FuelCart.fbx/Prefab/FuelCartPart/step_2_Wood", { DataType = "COMP_GROUNDED" })
-- mod:registerPrefabComponent("models/FuelCart.fbx/Prefab/FuelCartPart/step_3_Logs", { DataType = "COMP_GROUNDED" })
-- mod:registerPrefabComponent("models/FuelCart.fbx/Prefab/FuelCartPart/step_4_Coal", { DataType = "COMP_GROUNDED" })



-- Manual paths for buyer and seller (does not use building asset processor convention)
-- Couldn't get this code to work yet
-- mod:registerAssetId("models/FuelCart.fbx/Prefab/FuelCartPart/PATH_A_1", "FUELCART_PATHA")
-- mod:registerAssetId("models/FuelCart.fbx/Prefab/FuelCartPart/PATH_B_1", "FUELCART_PATHB")
-- mod:registerAssetId("models/FuelCart.fbx/Prefab/FuelCartPart/PATH_C_1", "FUELCART_PATHC")

-- mod:registerPrefabComponent("models/FuelCart.fbx/Prefab/FuelCartPart", {
--     DataType = "COMP_BUILDING_PART",
--     PathList = {
--         {
--             PathType = BUILDING_PATH_TYPE.WORK,
--             WayPointList = { "FUELCART_PATHA" }
--         },
--         {
--             PathType = BUILDING_PATH_TYPE.PICKUP,
--             WayPointList = { "FUELCART_PATHB" }
--         },
--         {
--             PathType = BUILDING_PATH_TYPE.PICKUP,
--             WayPointList = { "FUELCART_PATHC" }
--         },
--     }
-- })



-- Building: Fuel cart -------------------------------------------------------------------------------------
mod:register({
	DataType = "BUILDING",
	Id = "FUELCART",
	Name = "FUELCART_NAME",
	Description = "FUELCART_DESC",
    BuildingType = "GENERAL",
    AssetCoreBuildingPart = "FUELCART_PART"
})



-- Building part: Fuel cart -------------------------------------------------------------------------------------
mod:register({
	DataType = "BUILDING_PART",
	Id = "FUELCART_PART",
    BuildingFunction = "FUELCART_FUNCTION_ASSIGNABLE",
	BuildingZone = { 
        ZoneEntryList = {
            { 
                Polygon = polygon.createRectangle( {2, 3}, {0.0, 0.0} ),
                Type = { DEFAULT = true, NAVIGABLE = true, GRASS_CLEAR = true } 
            },
        }
    }, 
    ConstructionVisual = "FUELCART_PREFAB",
    ConstructorData = {
            DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
            CoreObjectPrefab = "FUELCART_PREFAB"
        }, 
	Cost = {
--		BuildRightTaxes = {	{ Resource = "GOLD", Quantity = 1 } },
		UpkeepCost = { { Resource = "GOLD", Quantity = 1 } },
		RessourcesNeeded = {
            { Resource = "WOOD", Quantity = 5 }
		}
    }
})



-- Building Function: Fuel cart: ASSIGNABLE -------------------------------------------------------------------------------------

mod:register({
    DataType = "BUILDING_FUNCTION_ASSIGNABLE",
    Id = "FUELCART_FUNCTION_ASSIGNABLE",
    AuthorizedFunctionList = "FUELCART_FUNCTION_LIST"
})

mod:register({
    DataType = "BUILDING_FUNCTION_LIST",
    Id = "FUELCART_FUNCTION_LIST",
    AssetFunctionList = {
        "FUELCART_FUNCTION_WOOD",
        "FUELCART_FUNCTION_COAL"
    }
})



-- Building Function: Fuel cart: Wood -------------------------------------------------------------------------------------

--[[
    By setting both the production parameters, and the market capacity, and by 
    setting WORK_BEHAVIOR instead of MARKET_TENDING_BEHAVIOR, the worker collects 
    wood from lumbercamps or village centers, rather than requiring this from the warehouse
]]

mod:register({
    DataType = "BUILDING_FUNCTION_MARKET",
    Id = "FUELCART_FUNCTION_WOOD",
    Name = "FUELCART_FUNCTION_WOOD_NAME",
    WorkerCapacity = 1,
    RelatedJob = { 
        Job = "FUELSELLER_JOB", 
--        Behavior = "MARKET_TENDING_BEHAVIOR" -- If this is set, she will not produce wood from wood, but will only try to sell it, and can only collect from a warehouse
        Behavior = "WORK_BEHAVIOR" -- required for worker to produce wood from wood
    },
    TypeList = {"FUEL_RESOURCE_TYPE_WOOD"}, 
    InputInventoryCapacity = {{ Resource = "WOOD", Quantity = 50 }}, -- these enable pseudo-production and collection of resources from non-warehouse locations
    ResourceListNeeded = {{ Resource = "WOOD", Quantity = 5 }}, -- these enable pseudo-production and collection of resources from non-warehouse locations
    ResourceProduced = {{ Resource = "WOOD", Quantity = 5 }}, -- these enable pseudo-production and collection of resources from non-warehouse locations

    IsOverrideCapacity = true, -- these enable selling
    CapacityPerSlot = 50 -- these enable selling
})



-- Building Function: Fuel cart: Coal -------------------------------------------------------------------------------------

mod:register({
    DataType = "BUILDING_FUNCTION_MARKET",
    Id = "FUELCART_FUNCTION_COAL",
    Name = "FUELCART_FUNCTION_COAL_NAME",
    WorkerCapacity = 1,
    RelatedJob = { 
        Job = "FUELSELLER_JOB", 
--        Behavior = "MARKET_TENDING_BEHAVIOR" -- If this is set, she will not produce wood from wood, but will only try to sell it
        Behavior = "WORK_BEHAVIOR" -- required for worker to produce wood from wood
    },
    TypeList = {"FUEL_RESOURCE_TYPE_COAL"}, 
    InputInventoryCapacity = {{ Resource = "COAL", Quantity = 50 }}, -- these enable pseudo-production and collection of resources from non-warehouse locations
    ResourceListNeeded = {{ Resource = "COAL", Quantity = 5 }}, -- these enable pseudo-production and collection of resources from non-warehouse locations
    ResourceProduced = {{ Resource = "COAL", Quantity = 5 }}, -- these enable pseudo-production and collection of resources from non-warehouse locations

    IsOverrideCapacity = true, -- these enable selling
    CapacityPerSlot = 50 -- these enable selling
})



