local mod = ...



-- Default Balancing (event list) ---------------------------------------------------------------------------------------------

mod:override({
    Id = "DEFAULT_BALANCING",
    EventList = {
        Action = "APPEND", -- by not appending, it should remove the native events like weather
        "EVENT_FUEL_START",
    }
})



-- Event: Start ---------------------------------------------------------------------------------------------

mod:register({
	DataType = "EVENT",
    Id = "EVENT_FUEL_START",
    IsInMainPool = false, -- the main pool seems to suppress itself until the first Weather event fires, so custom events should be outside the main pool
    IsRecurrent = false,
    DaysBetweenOccurences = nil,
    DaysToFirst = 1,
    Delay = 10, 
	ConditionList = {
		{
            DataType = "EVENT_CONDITION_BUILDING_BUILT",
            AssetBuilding = "MONUMENT_MARKET"
		}
	},
	ActionList = {
        {
            DataType = "EVENT_ACTION_GIVE_QUEST",
            Quest = "QUEST_FUEL_START"
        },
    }
})



-- Quest: Build ---------------------------------------------------------------------------------------------
mod:register({
    DataType = "QUEST",
    Id = "QUEST_FUEL_START",
    Name = "QUEST_FUEL_START_NAME",
    Description = "QUEST_FUEL_START_DESC",
    IsSilent = false,
	QuestFunction = {
        DataType = "QUEST_FUNCTION_BUILD", 
        BuildingListToBuild = {
            "FUELCART"
        }
    },
    QuestRewardList = {
        {
            DataType = "QUEST_REWARD_RESOURCES",
            ResourceCollection = {
                {
                    Resource = "WOOD",
                    Quantity = 20
                }
            }
        },
    },
    QuestFailList = nil,
    QuestListToUnlockAfter = nil,
    OnFailCallback = nil,
    OnSuccessCallback = {
        DataType = "EVENT_CALLBACK_TRIGGER_EVENT",
        Event = "EVENT_FUEL_BUILD",
        Delay = 0
    }
})



-- Event: Sell ---------------------------------------------------------------------------------------------

mod:register({
	DataType = "EVENT",
    Id = "EVENT_FUEL_BUILD",
    IsInMainPool = false, -- the main pool seems to suppress itself until the first Weather event fires, so custom events should be outside the main pool
    IsRecurrent = false,
    DaysBetweenOccurences = nil,
    DaysToFirst = 1,
    Delay = 10, 
	ConditionList = {
		{
            DataType = "EVENT_CONDITION_BUILDING_BUILT",
            AssetBuilding = "FUELCART"
		}
	},
	ActionList = {
        {
            DataType = "EVENT_ACTION_GIVE_QUEST",
            Quest = "QUEST_FUEL_SELL"
        },
    }
})



-- Quest: Sell ---------------------------------------------------------------------------------------------
mod:register({
    DataType = "QUEST",
    Id = "QUEST_FUEL_SELL",
    Name = "QUEST_FUEL_SELL_NAME",
    Description = "QUEST_FUEL_SELL_DESC",
    IsSilent = false,
	QuestFunction = {
        DataType = "QUEST_FUNCTION_CONTAINER", 
        QuestContainerDescription = "QUEST_FUEL_SELL_NAME",
        SubQuestList = {
            {
                DataType = "QUEST_FUNCTION_ASSIGN_RESOURCE_MARKET",
                Resource = "WOOD"
            },
            {
                DataType = "QUEST_FUNCTION_ASSIGN_WORKER",
                Job = "FUELSELLER_JOB"
            },
        }
    },
    QuestRewardList = nil,
    QuestFailList = nil,
    QuestListToUnlockAfter = nil,
    OnFailCallback = nil
})


